/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.contract;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import javax.enterprise.context.SessionScoped;
import javax.enterprise.event.Event;
import javax.inject.Inject;
import javax.inject.Named;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.kernel.tariff.option.common.bean.ContractTariffOption;
import ru.bitel.bgbilling.kernel.tariff.option.common.bean.TariffOption;
import ru.bitel.bgbilling.kernel.tariff.option.common.service.TariffOptionService;
import ru.bitel.common.Utils;
import ru.bitel.common.function.Async;
import ru.bitel.common.function.Lazy;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;

@SuppressWarnings("serial")
@Named
@SessionScoped
@BGInjection
public class TariffOptionBean
	extends AbstractBean
{
	
	@Inject
	Event<TariffOptionActivateEvent> event;

	@BGInject( module = false )
	private TariffOptionService tariffOptionService;
	
	@Inject
	private BalanceBean balanceBean;

	private Async<List<ContractTariffOption>> contractTariffOptionList;
	private Lazy<List<ContractTariffOption>> historyContractTariffOptionList;

	private Lazy<List<TariffOption>> tariffOptionList;
	private Async<List<TariffOption>> availableTariffOptionList;
	private Lazy<List<TariffOption>> availableTariffOptionListFilter;
	private String availableTariffFilter;

	@Override
	protected void init()
		throws BGException
	{
		super.init();

		populate();
	}

	public void populate()
		throws BGException
	{
		Date now = new Date();

		this.contractTariffOptionList = Async.of( ( ) -> tariffOptionService.contractTariffOptionListWeb( getContractId(), now, null ) );
		this.historyContractTariffOptionList = Lazy.of( ( ) -> {
			List<ContractTariffOption> history = tariffOptionService.contractTariffOptionHistory( getContractId(), now, true );
			if( history != null )
			{
				Collections.reverse( history );
			}
			return history;
		} );

		this.tariffOptionList = Lazy.of( ( ) -> Utils.maskNull( tariffOptionService.tariffOptionListAvailable( getContractId(), now, null, false, true ) ) );

		this.availableTariffOptionList = Async.of( ( ) -> {
			List<TariffOption> availableTariffOptionList = new ArrayList<TariffOption>();

			for( TariffOption o : this.tariffOptionList.get() )
			{
				if( o.isEnable() )
				{
					availableTariffOptionList.add( o );
				}
			}

			return availableTariffOptionList;
		} );
		
		setAvailableTariffFilter(null);
		
	}

	public List<TariffOption> getTariffOptionList()
		throws BGException
	{
		return tariffOptionList.get();
	}

	public List<TariffOption> getAvailableTariffOptionList()
		throws BGException
	{
		//return availableTariffOptionList.get();
	    return availableTariffOptionListFilter.get();
	}

	public List<ContractTariffOption> getContractTariffOptionList()
		throws BGException
	{
		return contractTariffOptionList.get();
	}

	public List<ContractTariffOption> getHistoryContractTariffOptionList()
		throws BGException
	{
		return historyContractTariffOptionList.get();
	}

	public void activate( int tariffOptionId, int tariffOptionActivateModeId )
		throws BGException
	{
		// бросаем событие активации ТО, если кто-то бросит исключение, процес активации прикратится.
		event.fire( new TariffOptionActivateEvent( getContractId(), tariffOptionId, tariffOptionActivateModeId ) );
		try
		{
			tariffOptionService.contractTariffOptionActivate( getContractId(), tariffOptionId, tariffOptionActivateModeId, true );
			
			addInfoMessage( null, "Тарифная опция успешно активирована" );
		}
		finally
		{
			try
			{
				balanceBean.populateCurrentContractBalance();
			}
			finally
			{
				populate();
			}
		}
	}

	public void deactivate( int contractTariffOptionId )
		throws BGException
	{
		try
		{
			tariffOptionService.contractTariffOptionDeactivate( getContractId(), contractTariffOptionId );
			
			addInfoMessage( null, "Тарифная опция успешно деактивирована" );
		}
		finally
		{
			populate();
		}
	}

	public void reactivate( int contractTariffOptionId )
		throws BGException
	{
		try
		{
			tariffOptionService.contractTariffOptionReactivate( getContractId(), contractTariffOptionId );
			
			addInfoMessage( null, "Тарифная опция успешно реактивирована" );
		}
		finally
		{
			populate();
		}
	}
	
	/** Событие активации ТО	 */
	public static class TariffOptionActivateEvent
	{
		private boolean stop = false;
		private int contractId;
		private int tariffOptionId;
		private int tariffOptionActivateModeId;

		public TariffOptionActivateEvent( int contractId, int tariffOptionId, int tariffOptionActivateModeId )
		{
			this.setContractId( contractId );
			this.tariffOptionId = tariffOptionId;
			this.tariffOptionActivateModeId = tariffOptionActivateModeId;
		}

		public boolean isStop()
		{
			return stop;
		}

		public void setStop( boolean stop )
		{
			this.stop = stop;
		}

		public int getTariffOptionId()
		{
			return tariffOptionId;
		}

		public void setTariffOptionId( int tariffOptionId )
		{
			this.tariffOptionId = tariffOptionId;
		}

		public int getTariffOptionActivateModeId()
		{
			return tariffOptionActivateModeId;
		}

		public void setTariffOptionActivateModeId( int tariffOptionActivateModeId )
		{
			this.tariffOptionActivateModeId = tariffOptionActivateModeId;
		}

		public int getContractId()
		{
			return contractId;
		}

		public void setContractId( int contractId )
		{
			this.contractId = contractId;
		}
		
	}

    public String getAvailableTariffFilter()
    {
        return availableTariffFilter;
    }

    public void setAvailableTariffFilter( String availableTariffFilter )
    {
        this.availableTariffFilter = availableTariffFilter;
        if(Utils.isEmptyString( availableTariffFilter) || availableTariffFilter.equals( "-1" ))
        {
            availableTariffOptionListFilter = Lazy.of( ()->availableTariffOptionList.get() );
        }
        else
        {
            Pattern p = Pattern.compile(availableTariffFilter);  
            availableTariffOptionListFilter = Lazy.of( ()->availableTariffOptionList.get().stream()
                                                       .filter( (i)->p.matcher( i.getTitle() ).matches() )
                                                       .collect(Collectors.toList()) );
            
        }
       
    }

    
}
